@@ -37,6 +37,14 @@
  * vorbisparse outputs have all of the metadata that oggmux expects to receive,
  * which allows you to (for example) remux an ogg/theora file.
  * </para>
+ * <para>
+ * In addition, this element allows you to fix badly synchronized streams. You
+ * pass in an array of (granule time, buffer time) synchronization points via
+ * the synchronization-points GValueArray property, and this element will adjust
+ * the granulepos values that it outputs. The adjustment will be made by
+ * offsetting all buffers that it outputs by a specified amount, and updating
+ * that offset from the value array whenever a keyframe is processed.
+ * </para>
  * <title>Example pipelines</title>
  * <para>
  * <programlisting>
@@ -64,6 +72,9 @@
 
 #include "gsttheoraparse.h"
 
+#define GST_CAT_DEFAULT theoraparse_debug
+GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
+
 static GstElementDetails theora_parse_details = {
   "TheoraParse",
   "Codec/Parser/Video",
@@ -85,9 +96,22 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_STATIC_CAPS ("video/x-theora")
     );
 
+enum
+{
+  PROP_0,
+  PROP_SYNCHRONIZATION_POINTS
+};
+
 GST_BOILERPLATE (GstTheoraParse, gst_theora_parse, GstElement,
     GST_TYPE_ELEMENT);
 
+static void theora_parse_dispose (GObject * object);
+static void theora_parse_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec);
+static void theora_parse_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec);
+
+static gboolean theora_parse_src_query (GstPad * pad, GstQuery * query);
 static GstFlowReturn theora_parse_chain (GstPad * pad, GstBuffer * buffer);
 static GstStateChangeReturn theora_parse_change_state (GstElement * element,
     GstStateChange transition);
@@ -109,9 +133,32 @@ gst_theora_parse_base_init (gpointer g_class)
 static void
 gst_theora_parse_class_init (GstTheoraParseClass * klass)
 {
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
 
+  gobject_class->dispose = theora_parse_dispose;
+  gobject_class->get_property = theora_parse_get_property;
+  gobject_class->set_property = theora_parse_set_property;
+
+  /**
+   * GstTheoraParse:sychronization-points
+   *
+   * An array of (granuletime, buffertime) pairs
+   *
+   * Since: 0.10.10
+   */
+  g_object_class_install_property (gobject_class, PROP_SYNCHRONIZATION_POINTS,
+      g_param_spec_value_array ("synchronization-points",
+          "Synchronization points",
+          "An array of (granuletime, buffertime) pairs",
+          g_param_spec_uint64 ("time", "Time",
+              "Time (either granuletime or buffertime)", 0, G_MAXUINT64, 0,
+              G_PARAM_READWRITE), (GParamFlags) G_PARAM_READWRITE));
+
   gstelement_class->change_state = theora_parse_change_state;
+
+  GST_DEBUG_CATEGORY_INIT (theoraparse_debug, "theoraparse", 0,
+      "Theora parser");
 }
 
 static void
@@ -129,6 +176,95 @@ gst_theora_parse_init (GstTheoraParse * parse, GstTheoraParseClass * g_class)
   gst_element_add_pad (GST_ELEMENT (parse), parse->srcpad);
 }
 
+static void
+theora_parse_dispose (GObject * object)
+{
+  GstTheoraParse *parse = GST_THEORA_PARSE (object);
+
+  g_free (parse->times);
+  parse->times = NULL;
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
+static void
+theora_parse_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec)
+{
+  GstTheoraParse *parse = GST_THEORA_PARSE (object);
+
+  switch (prop_id) {
+    case PROP_SYNCHRONIZATION_POINTS:
+    {
+      GValueArray *array;
+      guint i;
+
+      array = g_value_get_boxed (value);
+
+      if (array) {
+        if (array->n_values % 2)
+          goto odd_values;
+
+        g_free (parse->times);
+        parse->times = g_new (GstClockTime, array->n_values);
+        parse->npairs = array->n_values / 2;
+        for (i = 0; i < array->n_values; i++)
+          parse->times[i] = g_value_get_uint64 (&array->values[i]);
+      } else {
+        g_free (parse->times);
+        parse->npairs = 0;
+      }
+    }
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+
+  return;
+
+odd_values:
+  {
+    g_critical ("expected an even number of time values for "
+        "synchronization-points");
+    return;
+  }
+}
+
+static void
+theora_parse_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec)
+{
+  GstTheoraParse *parse = GST_THEORA_PARSE (object);
+
+  switch (prop_id) {
+    case PROP_SYNCHRONIZATION_POINTS:
+    {
+      GValueArray *array = NULL;
+      guint i;
+
+      array = g_value_array_new (parse->npairs * 2);
+
+      for (i = 0; i < parse->npairs; i++) {
+        GValue v = { 0, };
+
+        g_value_init (&v, G_TYPE_UINT64);
+        g_value_set_uint64 (&v, parse->times[i * 2]);
+        g_value_array_append (array, &v);
+        g_value_set_uint64 (&v, parse->times[i * 2 + 1]);
+        g_value_array_append (array, &v);
+        g_value_unset (&v);
+      }
+
+      g_value_set_boxed (value, array);
+    }
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
 static void
 theora_parse_set_header_on_caps (GstTheoraParse * parse, GstCaps * caps)
 {
@@ -177,6 +313,7 @@ theora_parse_set_streamheader (GstTheoraParse * parse)
 {
   GstCaps *caps;
   gint i;
+  guint32 bitstream_version;
 
   g_assert (!parse->streamheader_received);
 
@@ -205,14 +342,37 @@ theora_parse_set_streamheader (GstTheoraParse * parse)
   parse->fps_d = parse->info.fps_denominator;
   parse->shift = _theora_ilog (parse->info.keyframe_frequency_force - 1);
 
+  /* With libtheora-1.0beta1 the granulepos scheme was changed:
+   * where earlier the granulepos refered to the index/beginning
+   * of a frame, it now refers to the end, which matches the use
+   * in vorbis/speex. We check the bitstream version from the header so
+   * we know which way to interpret the incoming granuepos
+   */
+  bitstream_version = (parse->info.version_major << 16) |
+      (parse->info.version_minor << 8) | parse->info.version_subminor;
+  parse->is_old_bitstream = (bitstream_version <= 0x00030200);
+
   parse->streamheader_received = TRUE;
 }
 
+static void
+theora_parse_drain_event_queue (GstTheoraParse * parse)
+{
+  while (parse->event_queue->length) {
+    GstEvent *event;
+
+    event = GST_EVENT_CAST (g_queue_pop_head (parse->event_queue));
+    gst_pad_event_default (parse->sinkpad, event);
+  }
+}
+
 static void
 theora_parse_push_headers (GstTheoraParse * parse)
 {
   gint i;
 
+  theora_parse_drain_event_queue (parse);
+
   if (!parse->streamheader_received)
     theora_parse_set_streamheader (parse);
 
@@ -233,31 +393,45 @@ theora_parse_clear_queue (GstTheoraParse * parse)
     buf = GST_BUFFER_CAST (g_queue_pop_head (parse->buffer_queue));
     gst_buffer_unref (buf);
   }
+  while (parse->event_queue->length) {
+    GstEvent *event;
+
+    event = GST_EVENT_CAST (g_queue_pop_head (parse->event_queue));
+    gst_buffer_unref (event);
+  }
 }
 
 static gint64
-make_granulepos (gint64 keyframe, gint64 frame, gint shift)
+make_granulepos (GstTheoraParse * parse, gint64 keyframe, gint64 frame)
 {
   if (keyframe == -1)
     keyframe = 0;
+  /* If using newer theora, offset the granulepos by +1, see comment
+   * in theora_parse_set_streamheader */
+  if (!parse->is_old_bitstream)
+    keyframe += 1;
 
   g_return_val_if_fail (frame >= keyframe, -1);
-  g_return_val_if_fail (frame - keyframe < 1 << shift, -1);
+  g_return_val_if_fail (frame - keyframe < 1 << parse->shift, -1);
 
-  return (keyframe << shift) + (frame - keyframe);
+  return (keyframe << parse->shift) + (frame - keyframe);
 }
 
 static void
-parse_granulepos (gint64 granulepos, gint shift, gint64 * keyframe,
-    gint64 * frame)
+parse_granulepos (GstTheoraParse * parse, gint64 granulepos,
+    gint64 * keyframe, gint64 * frame)
 {
   gint64 kf;
 
-  kf = granulepos >> shift;
+  kf = granulepos >> parse->shift;
+  /* If using newer theora, offset the granulepos by -1, see comment
+   * in theora_parse_set_streamheader */
+  if (!parse->is_old_bitstream)
+    kf -= 1;
   if (keyframe)
     *keyframe = kf;
   if (frame)
-    *frame = kf + (granulepos & ((1 << shift) - 1));
+    *frame = kf + (granulepos & ((1 << parse->shift) - 1));
 }
 
 static gboolean
@@ -270,6 +444,42 @@ is_keyframe (GstBuffer * buf)
   return ((GST_BUFFER_DATA (buf)[0] & 0x40) == 0);
 }
 
+static void
+theora_parse_munge_granulepos (GstTheoraParse * parse, GstBuffer * buf,
+    gint64 keyframe, gint64 frame)
+{
+  gint64 frames_diff;
+  GstClockTimeDiff time_diff;
+
+  if (keyframe == frame) {
+    gint i;
+
+    /* update granule_offset */
+    for (i = 0; i < parse->npairs; i++) {
+      if (parse->times[i * 2] >= GST_BUFFER_OFFSET (buf))
+        break;
+    }
+    if (i > 0) {
+      /* time_diff gets reset below */
+      time_diff = parse->times[i * 2 - 1] - parse->times[i * 2 - 2];
+      parse->granule_offset = gst_util_uint64_scale (time_diff,
+          parse->fps_n, parse->fps_d * GST_SECOND);
+      parse->granule_offset <<= parse->shift;
+    }
+  }
+
+  frames_diff = parse->granule_offset >> parse->shift;
+  time_diff = gst_util_uint64_scale_int (GST_SECOND * frames_diff,
+      parse->fps_d, parse->fps_n);
+
+  GST_DEBUG_OBJECT (parse, "offsetting theora stream by %" G_GINT64_FORMAT
+      " frames (%" GST_TIME_FORMAT ")", frames_diff, GST_TIME_ARGS (time_diff));
+
+  GST_BUFFER_OFFSET_END (buf) += parse->granule_offset;
+  GST_BUFFER_OFFSET (buf) += time_diff;
+  GST_BUFFER_TIMESTAMP (buf) += time_diff;
+}
+
 static GstFlowReturn
 theora_parse_push_buffer (GstTheoraParse * parse, GstBuffer * buf,
     gint64 keyframe, gint64 frame)
@@ -283,13 +493,19 @@ theora_parse_push_buffer (GstTheoraParse * parse, GstBuffer * buf,
   next_time = gst_util_uint64_scale_int (GST_SECOND * (frame + 1),
       parse->fps_d, parse->fps_n);
 
-  GST_BUFFER_OFFSET_END (buf) = make_granulepos (keyframe, frame, parse->shift);
+  GST_BUFFER_OFFSET_END (buf) = make_granulepos (parse, keyframe, frame);
   GST_BUFFER_OFFSET (buf) = this_time;
   GST_BUFFER_TIMESTAMP (buf) = this_time;
   GST_BUFFER_DURATION (buf) = next_time - this_time;
 
   gst_buffer_set_caps (buf, GST_PAD_CAPS (parse->srcpad));
 
+  if (parse->times)
+    theora_parse_munge_granulepos (parse, buf, keyframe, frame);
+
+  GST_DEBUG_OBJECT (parse, "pushing buffer with granulepos %" G_GINT64_FORMAT
+      "|%" G_GINT64_FORMAT, keyframe, frame - keyframe);
+
   return gst_pad_push (parse->srcpad, buf);
 }
 
@@ -303,6 +519,12 @@ theora_parse_drain_queue_prematurely (GstTheoraParse * parse)
    * didn't-get-a-granulepos-on-the-last-packet case. Assuming a continuous
    * stream. */
 
+  GST_DEBUG_OBJECT (parse, "got EOS, draining queue");
+
+  /* if we get an eos before pushing the streamheaders, drain our events before
+   * eos */
+  theora_parse_drain_event_queue (parse);
+
   while (!g_queue_is_empty (parse->buffer_queue)) {
     GstBuffer *buf;
 
@@ -313,10 +535,12 @@ theora_parse_drain_queue_prematurely (GstTheoraParse * parse)
     if (is_keyframe (buf))
       /* we have a keyframe */
       parse->prev_keyframe = parse->prev_frame;
+    else
+      GST_BUFFER_FLAGS (buf) |= GST_BUFFER_FLAG_DELTA_UNIT;
 
     if (parse->prev_keyframe < 0) {
       if (GST_BUFFER_OFFSET_END_IS_VALID (buf)) {
-        parse_granulepos (GST_BUFFER_OFFSET_END (buf), parse->shift,
+        parse_granulepos (parse, GST_BUFFER_OFFSET_END (buf),
             &parse->prev_keyframe, NULL);
       } else {
         /* No previous keyframe known; can't extract one from this frame. That
@@ -343,12 +567,27 @@ static GstFlowReturn
 theora_parse_drain_queue (GstTheoraParse * parse, gint64 granulepos)
 {
   GstFlowReturn ret = GST_FLOW_OK;
-  gint64 keyframe, frame;
+  gint64 keyframe, prev_frame, frame;
 
-  parse_granulepos (granulepos, parse->shift, &keyframe, &frame);
+  parse_granulepos (parse, granulepos, &keyframe, &frame);
 
-  parse->prev_frame = MAX (parse->prev_frame,
-      frame - g_queue_get_length (parse->buffer_queue));
+  prev_frame = frame - g_queue_get_length (parse->buffer_queue);
+  if (prev_frame < parse->prev_frame) {
+    GST_WARNING ("jumped %" G_GINT64_FORMAT
+        " frames backwards! not sure what to do here",
+        parse->prev_frame - prev_frame);
+    ret = GST_FLOW_ERROR;
+    goto done;
+  } else if (prev_frame > parse->prev_frame) {
+    GST_INFO ("discontinuity detected (%" G_GINT64_FORMAT
+        " frames)", prev_frame - parse->prev_frame);
+    if (keyframe <= prev_frame && keyframe > parse->prev_keyframe)
+      parse->prev_keyframe = keyframe;
+    parse->prev_frame = prev_frame;
+  }
+
+  GST_DEBUG ("draining queue of length %d",
+      g_queue_get_length (parse->buffer_queue));
 
   while (!g_queue_is_empty (parse->buffer_queue)) {
     GstBuffer *buf;
@@ -361,6 +600,8 @@ theora_parse_drain_queue (GstTheoraParse * parse, gint64 granulepos)
     if (is_keyframe (buf))
       /* we have a keyframe */
       parse->prev_keyframe = parse->prev_frame;
+    else
+      GST_BUFFER_FLAGS (buf) |= GST_BUFFER_FLAG_DELTA_UNIT;
 
     ret = theora_parse_push_buffer (parse, buf, parse->prev_keyframe,
         parse->prev_frame);
@@ -384,7 +625,7 @@ theora_parse_queue_buffer (GstTheoraParse * parse, GstBuffer * buf)
 
   if (GST_BUFFER_OFFSET_END_IS_VALID (buf)) {
     if (parse->prev_keyframe < 0) {
-      parse_granulepos (GST_BUFFER_OFFSET_END (buf), parse->shift,
+      parse_granulepos (parse, GST_BUFFER_OFFSET_END (buf),
           &parse->prev_keyframe, NULL);
     }
     ret = theora_parse_drain_queue (parse, GST_BUFFER_OFFSET_END (buf));
@@ -422,6 +663,13 @@ theora_parse_chain (GstPad * pad, GstBuffer * buffer)
   return ret;
 }
 
+static gboolean
+theora_parse_queue_event (GstTheoraParse * parse, GstEvent * event)
+{
+  g_queue_push_tail (parse->event_queue, event);
+  return TRUE;
+}
+
 static gboolean
 theora_parse_sink_event (GstPad * pad, GstEvent * event)
 {
@@ -442,7 +690,10 @@ theora_parse_sink_event (GstPad * pad, GstEvent * event)
       ret = gst_pad_event_default (pad, event);
       break;
     default:
-      ret = gst_pad_event_default (pad, event);
+      if (parse->send_streamheader && GST_EVENT_IS_SERIALIZED (event))
+        ret = theora_parse_queue_event (parse, event);
+      else
+        ret = gst_pad_event_default (pad, event);
       break;
   }
 
@@ -631,8 +882,10 @@ theora_parse_change_state (GstElement * element, GstStateChange transition)
       parse->packetno = 0;
       parse->send_streamheader = TRUE;
       parse->buffer_queue = g_queue_new ();
+      parse->event_queue = g_queue_new ();
       parse->prev_keyframe = -1;
       parse->prev_frame = -1;
+      parse->granule_offset = 0;
       break;
     default:
       break;
@@ -646,6 +899,7 @@ theora_parse_change_state (GstElement * element, GstStateChange transition)
       theora_comment_clear (&parse->comment);
       theora_parse_clear_queue (parse);
       g_queue_free (parse->buffer_queue);
+      g_queue_free (parse->event_queue);
       parse->buffer_queue = NULL;
       for (i = 0; i < 3; i++) {
         if (parse->streamheader[i]) {
