@@ -169,6 +169,10 @@ gst_base_audio_src_class_init (GstBaseAudioSrcClass * klass)
   gstbasesrc_class->check_get_range =
       GST_DEBUG_FUNCPTR (gst_base_audio_src_check_get_range);
   gstbasesrc_class->fixate = GST_DEBUG_FUNCPTR (gst_base_audio_src_fixate);
+
+  /* ref class from a thread-safe context to work around missing bit of
+   * thread-safety in GObject */
+  g_type_class_ref (GST_TYPE_AUDIO_CLOCK);
 }
 
 static void
@@ -288,6 +292,52 @@ gst_base_audio_src_check_get_range (GstBaseSrc * bsrc)
   return TRUE;
 }
 
+/**
+ * gst_base_audio_src_set_provide_clock:
+ * @src: a #GstBaseAudioSrc
+ * @provide: new state
+ *
+ * Controls whether @src will provide a clock or not. If @provide is %TRUE, 
+ * gst_element_provide_clock() will return a clock that reflects the datarate
+ * of @src. If @provide is %FALSE, gst_element_provide_clock() will return NULL.
+ *
+ * Since: 0.10.16
+ */
+void
+gst_base_audio_src_set_provide_clock (GstBaseAudioSrc * src, gboolean provide)
+{
+  g_return_if_fail (GST_IS_BASE_AUDIO_SRC (src));
+
+  GST_OBJECT_LOCK (src);
+  src->priv->provide_clock = provide;
+  GST_OBJECT_UNLOCK (src);
+}
+
+/**
+ * gst_base_audio_src_get_provide_clock:
+ * @src: a #GstBaseAudioSrc
+ *
+ * Queries whether @src will provide a clock or not. See also
+ * gst_base_audio_src_set_provide_clock.
+ *
+ * Returns: %TRUE if @src will provide a clock.
+ *
+ * Since: 0.10.16
+ */
+gboolean
+gst_base_audio_src_get_provide_clock (GstBaseAudioSrc * src)
+{
+  gboolean result;
+
+  g_return_val_if_fail (GST_IS_BASE_AUDIO_SRC (src), FALSE);
+
+  GST_OBJECT_LOCK (src);
+  result = src->priv->provide_clock;
+  GST_OBJECT_UNLOCK (src);
+
+  return result;
+}
+
 static void
 gst_base_audio_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -304,9 +354,7 @@ gst_base_audio_src_set_property (GObject * object, guint prop_id,
       src->latency_time = g_value_get_int64 (value);
       break;
     case PROP_PROVIDE_CLOCK:
-      GST_OBJECT_LOCK (src);
-      src->priv->provide_clock = g_value_get_boolean (value);
-      GST_OBJECT_UNLOCK (src);
+      gst_base_audio_src_set_provide_clock (src, g_value_get_boolean (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -330,9 +378,7 @@ gst_base_audio_src_get_property (GObject * object, guint prop_id,
       g_value_set_int64 (value, src->latency_time);
       break;
     case PROP_PROVIDE_CLOCK:
-      GST_OBJECT_LOCK (src);
-      g_value_set_boolean (value, src->priv->provide_clock);
-      GST_OBJECT_UNLOCK (src);
+      g_value_set_boolean (value, gst_base_audio_src_get_provide_clock (src));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -600,9 +646,12 @@ gst_base_audio_src_create (GstBaseSrc * bsrc, guint64 offset, guint length,
       break;
 
     /* else something interrupted us and we wait for playing again. */
+    GST_DEBUG_OBJECT (src, "wait playing");
     if (gst_base_src_wait_playing (bsrc) != GST_FLOW_OK)
       goto stopped;
 
+    GST_DEBUG_OBJECT (src, "continue playing");
+
     /* read next samples */
     sample += read;
     samples -= read;
@@ -718,6 +767,7 @@ gst_base_audio_src_change_state (GstElement * element,
 
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
+      GST_DEBUG_OBJECT (src, "NULL->READY");
       if (src->ringbuffer == NULL) {
         src->ringbuffer = gst_base_audio_src_create_ringbuffer (src);
       }
@@ -725,13 +775,22 @@ gst_base_audio_src_change_state (GstElement * element,
         goto open_failed;
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
+      GST_DEBUG_OBJECT (src, "READY->PAUSED");
       src->next_sample = -1;
       gst_ring_buffer_set_flushing (src->ringbuffer, FALSE);
+      gst_ring_buffer_may_start (src->ringbuffer, FALSE);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
+      GST_DEBUG_OBJECT (src, "PAUSED->PLAYING");
       gst_ring_buffer_may_start (src->ringbuffer, TRUE);
       break;
+    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
+      GST_DEBUG_OBJECT (src, "PLAYING->PAUSED");
+      gst_ring_buffer_may_start (src->ringbuffer, FALSE);
+      gst_ring_buffer_pause (src->ringbuffer);
+      break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+      GST_DEBUG_OBJECT (src, "PAUSED->READY");
       gst_ring_buffer_set_flushing (src->ringbuffer, TRUE);
       break;
     default:
@@ -741,14 +800,12 @@ gst_base_audio_src_change_state (GstElement * element,
   ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
   switch (transition) {
-    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-      gst_ring_buffer_may_start (src->ringbuffer, FALSE);
-      gst_ring_buffer_pause (src->ringbuffer);
-      break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+      GST_DEBUG_OBJECT (src, "PAUSED->READY");
       gst_ring_buffer_release (src->ringbuffer);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
+      GST_DEBUG_OBJECT (src, "READY->NULL");
       gst_ring_buffer_close_device (src->ringbuffer);
       gst_object_unparent (GST_OBJECT_CAST (src->ringbuffer));
       src->ringbuffer = NULL;
