@@ -42,10 +42,10 @@ gst_audio_check_channel_positions (const GstAudioChannelPosition * pos,
     gint channels)
 {
   gint i, n;
-  struct
+  const struct
   {
-    GstAudioChannelPosition pos1[2];
-    GstAudioChannelPosition pos2[1];
+    const GstAudioChannelPosition pos1[2];
+    const GstAudioChannelPosition pos2[1];
   } conf[] = {
     /* front: mono <-> stereo */
     { {
@@ -65,14 +65,32 @@ gst_audio_check_channel_positions (const GstAudioChannelPosition * pos,
     GST_AUDIO_CHANNEL_POSITION_INVALID}}
   };
 
+  g_assert (pos != NULL && channels > 0);
+
   /* check for invalid channel positions */
   for (n = 0; n < channels; n++) {
-    if (pos[n] == GST_AUDIO_CHANNEL_POSITION_INVALID) {
-      g_warning ("Position %d is invalid, not allowed", n);
+    if (pos[n] <= GST_AUDIO_CHANNEL_POSITION_INVALID ||
+        pos[n] >= GST_AUDIO_CHANNEL_POSITION_NUM) {
+      g_warning ("Channel position %d is invalid, not allowed", n);
       return FALSE;
     }
   }
 
+  /* either all channel positions are NONE or all are defined,
+   * but having only some channel positions NONE and others not
+   * is not allowed */
+  if (pos[0] == GST_AUDIO_CHANNEL_POSITION_NONE) {
+    for (n = 1; n < channels; ++n) {
+      if (pos[n] != GST_AUDIO_CHANNEL_POSITION_NONE) {
+        g_warning ("Either all channel positions must be defined, or all "
+            "be set to NONE, having only some defined is not allowed");
+        return FALSE;
+      }
+    }
+    /* all positions are NONE, we are done here */
+    return TRUE;
+  }
+
   /* check for multiple position occurrences */
   for (i = GST_AUDIO_CHANNEL_POSITION_INVALID + 1;
       i < GST_AUDIO_CHANNEL_POSITION_NUM; i++) {
@@ -83,6 +101,13 @@ gst_audio_check_channel_positions (const GstAudioChannelPosition * pos,
         count++;
     }
 
+    /* NONE may not occur mixed with other channel positions */
+    if (i == GST_AUDIO_CHANNEL_POSITION_NONE && count > 0) {
+      g_warning ("Either all channel positions must be defined, or all "
+          "be set to NONE, having only some defined is not allowed");
+      return FALSE;
+    }
+
     if (count > 1) {
       g_warning ("Channel position %d occurred %d times, not allowed",
           i, count);
@@ -108,11 +133,12 @@ gst_audio_check_channel_positions (const GstAudioChannelPosition * pos,
     }
   }
 
-  /* 0.8.x evilry */
-  if ((channels == 1 && pos[0] != GST_AUDIO_CHANNEL_POSITION_FRONT_MONO) ||
-      (channels == 2 && (pos[0] != GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT ||
-              pos[1] != GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT))) {
-    g_warning ("0.8.x: channels=1 implies mono; channels=2 implies stereo");
+  /* Throw warning if we encounter an unusual 2-channel configuration,
+   * at least until someone finds a reason why we should not */
+  if (channels == 2 && (pos[0] != GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT ||
+          pos[1] != GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT)) {
+    g_warning ("channels=2 implies stereo, but channel positions are "
+        "< %d, %d>", pos[0], pos[1]);
     return FALSE;
   }
 
@@ -490,10 +516,10 @@ gst_audio_fixate_channel_positions (GstStructure * str)
    * either one until all unknown values are filled. This might not work in
    * corner cases but should work OK for the general case.
    */
-  struct
+  const struct
   {
-    GstAudioChannelPosition pos1[2];
-    GstAudioChannelPosition pos2[1];
+    const GstAudioChannelPosition pos1[2];
+    const GstAudioChannelPosition pos2[1];
   } conf[] = {
     /* front: mono <-> stereo */
     { {
