@@ -24,7 +24,7 @@
 #endif
 
 #include "mixertrack.h"
-
+#ifndef GST_DISABLE_DEPRECATED
 enum
 {
   /* FILL ME */
@@ -33,11 +33,14 @@ enum
   SIGNAL_MUTE_TOGGLED,
   LAST_SIGNAL
 };
+static guint signals[LAST_SIGNAL] = { 0 };
+#endif
 
 enum
 {
   ARG_0,
   ARG_LABEL,
+  ARG_UNTRANSLATED_LABEL,
   ARG_MIN_VOLUME,
   ARG_MAX_VOLUME,
   ARG_FLAGS,
@@ -50,9 +53,10 @@ static void gst_mixer_track_dispose (GObject * object);
 
 static void gst_mixer_track_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
+static void gst_mixer_track_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec);
 
 static GObjectClass *parent_class = NULL;
-static guint signals[LAST_SIGNAL] = { 0 };
 
 GType
 gst_mixer_track_get_type (void)
@@ -89,10 +93,28 @@ gst_mixer_track_class_init (GstMixerTrackClass * klass)
   parent_class = g_type_class_peek_parent (klass);
 
   object_klass->get_property = gst_mixer_track_get_property;
+  object_klass->set_property = gst_mixer_track_set_property;
 
   g_object_class_install_property (object_klass, ARG_LABEL,
       g_param_spec_string ("label", "Track label",
-          "The label assigned to the track", NULL, G_PARAM_READABLE));
+          "The label assigned to the track (may be translated)", NULL,
+          G_PARAM_READABLE));
+
+  /**
+   * GstMixerTrack:untranslated-label
+   *
+   * The untranslated label of the mixer track, if available. Mixer track
+   * implementations must set this at construct time. Applications may find
+   * this useful to determine icons for various kind of tracks. However,
+   * applications mustn't make any assumptions about the naming of tracks,
+   * the untranslated labels are purely informational and may change.
+   *
+   * Since: 0.10.13
+   **/
+  g_object_class_install_property (object_klass, ARG_UNTRANSLATED_LABEL,
+      g_param_spec_string ("untranslated-label", "Untranslated track label",
+          "The untranslated label assigned to the track (since 0.10.13)",
+          NULL, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
   g_object_class_install_property (object_klass, ARG_MIN_VOLUME,
       g_param_spec_int ("min_volume", "Minimum volume level",
@@ -114,6 +136,7 @@ gst_mixer_track_class_init (GstMixerTrackClass * klass)
           "The number of channels contained within the track",
           0, G_MAXINT, 0, G_PARAM_READABLE));
 
+#if 0
   signals[SIGNAL_RECORD_TOGGLED] =
       g_signal_new ("record_toggled", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST,
@@ -135,6 +158,7 @@ gst_mixer_track_class_init (GstMixerTrackClass * klass)
           volume_changed),
       NULL, NULL, g_cclosure_marshal_VOID__POINTER,
       G_TYPE_NONE, 1, G_TYPE_POINTER);
+#endif
 
   object_klass->dispose = gst_mixer_track_dispose;
 }
@@ -148,6 +172,9 @@ gst_mixer_track_init (GstMixerTrack * mixer_track)
   mixer_track->num_channels = 0;
 }
 
+/* FIXME 0.11: move this as a member into the mixer track structure */
+#define MIXER_TRACK_OBJECT_DATA_KEY_UNTRANSLATED_LABEL "gst-mixer-track-ulabel"
+
 static void
 gst_mixer_track_get_property (GObject * object, guint prop_id, GValue * value,
     GParamSpec * pspec)
@@ -160,6 +187,11 @@ gst_mixer_track_get_property (GObject * object, guint prop_id, GValue * value,
     case ARG_LABEL:
       g_value_set_string (value, mixer_track->label);
       break;
+    case ARG_UNTRANSLATED_LABEL:
+      g_value_set_string (value,
+          (const gchar *) g_object_get_data (G_OBJECT (mixer_track),
+              MIXER_TRACK_OBJECT_DATA_KEY_UNTRANSLATED_LABEL));
+      break;
     case ARG_MIN_VOLUME:
       g_value_set_int (value, mixer_track->min_volume);
       break;
@@ -178,6 +210,26 @@ gst_mixer_track_get_property (GObject * object, guint prop_id, GValue * value,
   }
 }
 
+static void
+gst_mixer_track_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec)
+{
+  GstMixerTrack *mixer_track;
+
+  mixer_track = GST_MIXER_TRACK (object);
+
+  switch (prop_id) {
+    case ARG_UNTRANSLATED_LABEL:
+      g_object_set_data_full (G_OBJECT (mixer_track),
+          MIXER_TRACK_OBJECT_DATA_KEY_UNTRANSLATED_LABEL,
+          g_value_dup_string (value), (GDestroyNotify) g_free);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
 static void
 gst_mixer_track_dispose (GObject * object)
 {
