@@ -111,8 +111,6 @@ struct _GstTagDemuxPrivate
   GstSegment segment;
   gboolean need_newseg;
   gboolean newseg_update;
-
-  GList *pending_events;
 };
 
 /* Require at least 8kB of data before we attempt typefind. 
@@ -165,23 +163,6 @@ static void gst_tag_demux_init (GstTagDemux * obj, GstTagDemuxClass * klass);
 
 static gpointer parent_class;   /* NULL */
 
-GType
-gst_tag_demux_result_get_type (void)
-{
-  static GType etype = 0;
-  if (etype == 0) {
-    static const GEnumValue values[] = {
-      {GST_TAG_DEMUX_RESULT_BROKEN_TAG, "GST_TAG_DEMUX_RESULT_BROKEN_TAG",
-          "broken-tag"},
-      {GST_TAG_DEMUX_RESULT_AGAIN, "GST_TAG_DEMUX_RESULT_AGAIN", "again"},
-      {GST_TAG_DEMUX_RESULT_OK, "GST_TAG_DEMUX_RESULT_OK", "ok"},
-      {0, NULL, NULL}
-    };
-    etype = g_enum_register_static ("GstTagDemuxResult", values);
-  }
-  return etype;
-}
-
 /* Cannot use boilerplate macros here because we want the abstract flag */
 GType
 gst_tag_demux_get_type (void)
@@ -269,11 +250,6 @@ gst_tag_demux_reset (GstTagDemux * tagdemux)
   gst_segment_init (&tagdemux->priv->segment, GST_FORMAT_UNDEFINED);
   tagdemux->priv->need_newseg = TRUE;
   tagdemux->priv->newseg_update = FALSE;
-
-  g_list_foreach (tagdemux->priv->pending_events,
-      (GFunc) gst_mini_object_unref, NULL);
-  g_list_free (tagdemux->priv->pending_events);
-  tagdemux->priv->pending_events = NULL;
 }
 
 static void
@@ -314,6 +290,8 @@ gst_tag_demux_dispose (GObject * object)
 static gboolean
 gst_tag_demux_add_srcpad (GstTagDemux * tagdemux, GstCaps * new_caps)
 {
+  GstPad *srcpad = NULL;
+
   if (tagdemux->priv->src_caps == NULL ||
       !gst_caps_is_equal (new_caps, tagdemux->priv->src_caps)) {
 
@@ -330,7 +308,7 @@ gst_tag_demux_add_srcpad (GstTagDemux * tagdemux, GstCaps * new_caps)
   }
 
   if (tagdemux->priv->srcpad == NULL) {
-    tagdemux->priv->srcpad =
+    srcpad = tagdemux->priv->srcpad =
         gst_pad_new_from_template (gst_element_class_get_pad_template
         (GST_ELEMENT_GET_CLASS (tagdemux), "src"), "src");
     g_return_val_if_fail (tagdemux->priv->srcpad != NULL, FALSE);
@@ -437,6 +415,8 @@ gst_tag_demux_trim_buffer (GstTagDemux * tagdemux, GstBuffer ** buf_ref)
     need_sub = TRUE;
   }
 
+  g_assert (out_size > 0);
+
   if (need_sub == TRUE) {
     if (out_size != GST_BUFFER_SIZE (buf) || !gst_buffer_is_writable (buf)) {
       GstBuffer *sub;
@@ -660,6 +640,11 @@ gst_tag_demux_chain (GstPad * pad, GstBuffer * buf)
     case GST_TAG_DEMUX_STREAMING:{
       GstBuffer *outbuf = NULL;
 
+      if (demux->priv->send_tag_event) {
+        gst_tag_demux_send_tag_event (demux);
+        demux->priv->send_tag_event = FALSE;
+      }
+
       /* Trim the buffer and adjust offset */
       if (demux->priv->collect) {
         outbuf = demux->priv->collect;
@@ -668,13 +653,19 @@ gst_tag_demux_chain (GstPad * pad, GstBuffer * buf)
           return GST_FLOW_UNEXPECTED;
       }
       if (outbuf) {
-        GList *events;
-
         if (G_UNLIKELY (demux->priv->srcpad == NULL)) {
           gst_buffer_unref (outbuf);
           return GST_FLOW_ERROR;
         }
 
+        GST_DEBUG_OBJECT (demux, "Pushing buffer %p", outbuf);
+        /* gst_util_dump_mem (GST_BUFFER_DATA (outbuf),
+           GST_BUFFER_SIZE (outbuf)); */
+
+        /* Ensure the caps are set correctly */
+        outbuf = gst_buffer_make_metadata_writable (outbuf);
+        gst_buffer_set_caps (outbuf, GST_PAD_CAPS (demux->priv->srcpad));
+
         /* Might need a new segment before the buffer */
         if (demux->priv->need_newseg) {
           if (!gst_tag_demux_send_new_segment (demux)) {
@@ -684,31 +675,6 @@ gst_tag_demux_chain (GstPad * pad, GstBuffer * buf)
           demux->priv->need_newseg = FALSE;
         }
 
-        /* send any pending events we cached */
-        GST_OBJECT_LOCK (demux);
-        events = demux->priv->pending_events;
-        demux->priv->pending_events = NULL;
-        GST_OBJECT_UNLOCK (demux);
-
-        while (events != NULL) {
-          GST_DEBUG_OBJECT (demux->priv->srcpad, "sending cached %s event: %"
-              GST_PTR_FORMAT, GST_EVENT_TYPE_NAME (events->data), events->data);
-          gst_pad_push_event (demux->priv->srcpad, GST_EVENT (events->data));
-          events = g_list_delete_link (events, events);
-        }
-
-        /* Send our own pending tag event */
-        if (demux->priv->send_tag_event) {
-          gst_tag_demux_send_tag_event (demux);
-          demux->priv->send_tag_event = FALSE;
-        }
-
-        /* Ensure the caps are set correctly */
-        outbuf = gst_buffer_make_metadata_writable (outbuf);
-        gst_buffer_set_caps (outbuf, GST_PAD_CAPS (demux->priv->srcpad));
-
-        GST_LOG_OBJECT (demux, "Pushing buffer %p", outbuf);
-
         return gst_pad_push (demux->priv->srcpad, outbuf);
       }
     }
@@ -754,23 +720,8 @@ gst_tag_demux_sink_event (GstPad * pad, GstEvent * event)
       ret = TRUE;
       break;
     }
-    case GST_EVENT_FLUSH_STOP:
-    case GST_EVENT_FLUSH_START:
-      ret = gst_pad_event_default (pad, event);
-      break;
     default:
-      if (demux->priv->need_newseg && GST_EVENT_IS_SERIALIZED (event)) {
-        /* Cache all events if we have a pending segment, so they don't get
-         * lost (esp. tag events) */
-        GST_INFO_OBJECT (demux, "caching event: %" GST_PTR_FORMAT, event);
-        GST_OBJECT_LOCK (demux);
-        demux->priv->pending_events =
-            g_list_append (demux->priv->pending_events, event);
-        GST_OBJECT_UNLOCK (demux);
-        ret = TRUE;
-      } else {
-        ret = gst_pad_event_default (pad, event);
-      }
+      ret = gst_pad_event_default (pad, event);
       break;
   }
 
@@ -871,14 +822,12 @@ gst_tag_demux_srcpad_event (GstPad * pad, GstEvent * event)
       break;
     }
     default:
-      res = gst_pad_push_event (tagdemux->priv->sinkpad, event);
-      event = NULL;
+      /* FIXME: shouldn't we pass unknown and unhandled events upstream? */
       break;
   }
 
   gst_object_unref (tagdemux);
-  if (event)
-    gst_event_unref (event);
+  gst_event_unref (event);
   return res;
 }
 
@@ -1203,13 +1152,6 @@ gst_tag_demux_sink_activate (GstPad * sinkpad)
     demux->priv->send_tag_event = TRUE;
   }
 
-  if (demux->priv->upstream_size <=
-      demux->priv->strip_start + demux->priv->strip_end) {
-    /* There was no data (probably due to a truncated file) */
-    GST_DEBUG_OBJECT (demux, "No data in file");
-    return FALSE;
-  }
-
   /* 3 - Do typefinding on data */
   caps = gst_type_find_helper_get_range (GST_OBJECT (demux),
       (GstTypeFindHelperGetRangeFunction) gst_tag_demux_read_range,
@@ -1320,7 +1262,7 @@ read_beyond_end:
   {
     GST_DEBUG_OBJECT (demux, "attempted read beyond end of file");
     if (*buffer != NULL) {
-      gst_buffer_unref (*buffer);
+      gst_buffer_unref (buffer);
       *buffer = NULL;
     }
     return GST_FLOW_UNEXPECTED;
