@@ -29,6 +29,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
+#include <string.h>             /* memset, in FD_ZERO macro */
 #include <unistd.h>
 #include <sys/ioctl.h>
 
@@ -110,13 +111,15 @@ gst_tcp_socket_write (int socket, const void *buf, size_t count)
         count - bytes_written, MSG_NOSIGNAL);
 
     if (wrote <= 0) {
-      GST_WARNING ("error while writing");
       return bytes_written;
     }
     bytes_written += wrote;
   }
 
-  GST_LOG ("wrote %" G_GSIZE_FORMAT " bytes successfully", bytes_written);
+  if (bytes_written < 0)
+    GST_WARNING ("error while writing");
+  else
+    GST_LOG ("wrote %" G_GSIZE_FORMAT " bytes succesfully", bytes_written);
   return bytes_written;
 }
 
@@ -125,24 +128,30 @@ gst_tcp_socket_write (int socket, const void *buf, size_t count)
  */
 static GstFlowReturn
 gst_tcp_socket_read (GstElement * this, int socket, void *buf, size_t count,
-    GstPoll * fdset)
+    int cancel_fd)
 {
+  fd_set testfds;
+  int maxfdp1;
   ssize_t n;
   size_t bytes_read;
   int num_to_read;
-  int ret;
 
   bytes_read = 0;
 
   while (bytes_read < count) {
     /* do a blocking select on the socket */
+    FD_ZERO (&testfds);
+    FD_SET (socket, &testfds);
+    if (cancel_fd >= 0)
+      FD_SET (cancel_fd, &testfds);
+    maxfdp1 = MAX (socket, cancel_fd) + 1;
+
     /* no action (0) is an error too in our case */
-    if ((ret = gst_poll_wait (fdset, GST_CLOCK_TIME_NONE)) <= 0) {
-      if (ret == -1 && errno == EBUSY)
-        goto cancelled;
-      else
-        goto select_error;
-    }
+    if (select (maxfdp1, &testfds, NULL, NULL, 0) <= 0)
+      goto select_error;
+
+    if (cancel_fd >= 0 && FD_ISSET (cancel_fd, &testfds))
+      goto cancelled;
 
     /* ask how much is available for reading on the socket */
     if (ioctl (socket, FIONREAD, &num_to_read) < 0)
@@ -207,12 +216,10 @@ short_read:
 
 /* close the socket and reset the fd.  Used to clean up after errors. */
 void
-gst_tcp_socket_close (GstPollFD * socket)
+gst_tcp_socket_close (int *socket)
 {
-  if (socket->fd >= 0) {
-    close (socket->fd);
-    socket->fd = -1;
-  }
+  close (*socket);
+  *socket = -1;
 }
 
 /* read a buffer from the given socket
@@ -222,26 +229,33 @@ gst_tcp_socket_close (GstPollFD * socket)
  *         EOS
  */
 GstFlowReturn
-gst_tcp_read_buffer (GstElement * this, int socket, GstPoll * fdset,
+gst_tcp_read_buffer (GstElement * this, int socket, int cancel_fd,
     GstBuffer ** buf)
 {
+  fd_set testfds;
   int ret;
+  int maxfdp1;
   ssize_t bytes_read;
   int readsize;
 
   *buf = NULL;
 
   /* do a blocking select on the socket */
+  FD_ZERO (&testfds);
+  FD_SET (socket, &testfds);
+  if (cancel_fd >= 0)
+    FD_SET (cancel_fd, &testfds);
+  maxfdp1 = MAX (socket, cancel_fd) + 1;
+
   /* no action (0) is an error too in our case */
-  if ((ret = gst_poll_wait (fdset, GST_CLOCK_TIME_NONE)) <= 0) {
-    if (ret == -1 && errno == EBUSY)
-      goto cancelled;
-    else
-      goto select_error;
-  }
+  if ((ret = select (maxfdp1, &testfds, NULL, NULL, 0)) <= 0)
+    goto select_error;
+
+  if (cancel_fd >= 0 && FD_ISSET (cancel_fd, &testfds))
+    goto cancelled;
 
   /* ask how much is available for reading on the socket */
-  if (ioctl (socket, FIONREAD, &readsize) < 0)
+  if ((ret = ioctl (socket, FIONREAD, &readsize)) < 0)
     goto ioctl_error;
 
   if (readsize == 0)
@@ -284,7 +298,7 @@ ioctl_error:
 got_eos:
   {
     GST_DEBUG_OBJECT (this, "Got EOS on socket stream");
-    return GST_FLOW_UNEXPECTED;
+    return GST_FLOW_WRONG_STATE;
   }
 read_error:
   {
@@ -312,7 +326,7 @@ short_read:
  *         EOS
  */
 GstFlowReturn
-gst_tcp_gdp_read_buffer (GstElement * this, int socket, GstPoll * fdset,
+gst_tcp_gdp_read_buffer (GstElement * this, int socket, int cancel_fd,
     GstBuffer ** buf)
 {
   GstFlowReturn ret;
@@ -324,7 +338,8 @@ gst_tcp_gdp_read_buffer (GstElement * this, int socket, GstPoll * fdset,
   *buf = NULL;
   header = g_malloc (GST_DP_HEADER_LENGTH);
 
-  ret = gst_tcp_socket_read (this, socket, header, GST_DP_HEADER_LENGTH, fdset);
+  ret = gst_tcp_socket_read (this, socket, header, GST_DP_HEADER_LENGTH,
+      cancel_fd);
 
   if (ret != GST_FLOW_OK)
     goto header_read_error;
@@ -342,7 +357,7 @@ gst_tcp_gdp_read_buffer (GstElement * this, int socket, GstPoll * fdset,
   g_free (header);
 
   ret = gst_tcp_socket_read (this, socket, GST_BUFFER_DATA (*buf),
-      GST_BUFFER_SIZE (*buf), fdset);
+      GST_BUFFER_SIZE (*buf), cancel_fd);
 
   if (ret != GST_FLOW_OK)
     goto data_read_error;
@@ -379,7 +394,7 @@ data_read_error:
 }
 
 GstFlowReturn
-gst_tcp_gdp_read_caps (GstElement * this, int socket, GstPoll * fdset,
+gst_tcp_gdp_read_caps (GstElement * this, int socket, int cancel_fd,
     GstCaps ** caps)
 {
   GstFlowReturn ret;
@@ -393,7 +408,8 @@ gst_tcp_gdp_read_caps (GstElement * this, int socket, GstPoll * fdset,
   *caps = NULL;
   header = g_malloc (GST_DP_HEADER_LENGTH);
 
-  ret = gst_tcp_socket_read (this, socket, header, GST_DP_HEADER_LENGTH, fdset);
+  ret = gst_tcp_socket_read (this, socket, header, GST_DP_HEADER_LENGTH,
+      cancel_fd);
 
   if (ret != GST_FLOW_OK)
     goto header_read_error;
@@ -413,7 +429,7 @@ gst_tcp_gdp_read_caps (GstElement * this, int socket, GstPoll * fdset,
       "Reading %" G_GSIZE_FORMAT " bytes for caps packet payload",
       payload_length);
 
-  ret = gst_tcp_socket_read (this, socket, payload, payload_length, fdset);
+  ret = gst_tcp_socket_read (this, socket, payload, payload_length, cancel_fd);
 
   if (ret != GST_FLOW_OK)
     goto payload_read_error;
