@@ -42,7 +42,7 @@ enum
   LAST_SIGNAL
 };
 
-GST_DEBUG_CATEGORY (tcpclientsink_debug);
+GST_DEBUG_CATEGORY_STATIC (tcpclientsink_debug);
 #define GST_CAT_DEFAULT (tcpclientsink_debug)
 
 enum
@@ -136,10 +136,10 @@ gst_tcp_client_sink_class_init (GstTCPClientSink * klass)
   gobject_class->get_property = gst_tcp_client_sink_get_property;
   gobject_class->finalize = gst_tcp_client_sink_finalize;
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_HOST,
+  g_object_class_install_property (gobject_class, ARG_HOST,
       g_param_spec_string ("host", "Host", "The host/IP to send the packets to",
           TCP_DEFAULT_HOST, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_PORT,
+  g_object_class_install_property (gobject_class, ARG_PORT,
       g_param_spec_int ("port", "Port", "The port to send the packets to",
           0, TCP_HIGHEST_PORT, TCP_DEFAULT_PORT, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_PROTOCOL,
@@ -171,6 +171,8 @@ gst_tcp_client_sink_finalize (GObject * gobject)
   GstTCPClientSink *this = GST_TCP_CLIENT_SINK (gobject);
 
   g_free (this->host);
+
+  G_OBJECT_CLASS (parent_class)->finalize (gobject);
 }
 
 static gboolean
@@ -266,7 +268,7 @@ write_error:
   {
     GST_ELEMENT_ERROR (sink, RESOURCE, WRITE,
         (_("Error while sending data to \"%s:%d\"."), sink->host, sink->port),
-        ("Only %d of %d bytes written: %s",
+        ("Only %" G_GSIZE_FORMAT " of %u bytes written: %s",
             wrote, GST_BUFFER_SIZE (buf), g_strerror (errno)));
     return GST_FLOW_ERROR;
   }
