@@ -47,10 +47,10 @@ G_BEGIN_DECLS
 #define GST_TCP_SERVER_SINK(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_TCP_SERVER_SINK,GstTCPServerSink))
 #define GST_TCP_SERVER_SINK_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_TCP_SERVER_SINK,GstTCPServerSink))
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_TCP_SERVER_SINK,GstTCPServerSinkClass))
 #define GST_IS_TCP_SERVER_SINK(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_TCP_SERVER_SINK))
-#define GST_IS_TCP_SERVER_SINK_CLASS(obj) \
+#define GST_IS_TCP_SERVER_SINK_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_TCP_SERVER_SINK))
 
 typedef struct _GstTCPServerSink GstTCPServerSink;
@@ -62,6 +62,11 @@ typedef enum {
   GST_TCP_SERVER_SINK_FLAG_LAST        = (GST_ELEMENT_FLAG_LAST << 2)
 } GstTCPServerSinkFlags;
 
+/**
+ * GstTCPServerSink:
+ *
+ * Opaque data structure.
+ */
 struct _GstTCPServerSink {
   GstMultiFdSink element;
 
