@@ -21,6 +21,11 @@
  */
 
 #include <gst/check/gstcheck.h>
+#include <gst/check/gstbufferstraw.h>
+
+#include <theora/theora.h>
+
+#ifndef GST_DISABLE_PARSE
 
 #define TIMESTAMP_OFFSET G_GINT64_CONSTANT(3249870963)
 #define FRAMERATE 10
@@ -28,135 +33,47 @@
 /* I know all of these have a shift of 6 bits */
 #define GRANULEPOS_SHIFT 6
 
-static GCond *cond = NULL;
-static GMutex *lock = NULL;
-static GstBuffer *buf = NULL;
-static gulong id;
 
-/* called for every buffer.  Waits until the global "buf" variable is unset,
- * then sets it to the buffer received, and signals. */
-static gboolean
-buffer_probe (GstPad * pad, GstBuffer * buffer, gpointer unused)
-{
-  g_mutex_lock (lock);
+#define check_buffer_is_header(buffer,is_header) \
+  fail_unless (GST_BUFFER_FLAG_IS_SET (buffer,   \
+          GST_BUFFER_FLAG_IN_CAPS) == is_header, \
+      "GST_BUFFER_IN_CAPS is set to %d but expected %d", \
+      GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_IN_CAPS), is_header)
 
-  while (buf != NULL)
-    g_cond_wait (cond, lock);
+#define check_buffer_timestamp(buffer,timestamp) \
+  fail_unless (GST_BUFFER_TIMESTAMP (buffer) == timestamp, \
+      "expected timestamp %" GST_TIME_FORMAT \
+      ", but got timestamp %" GST_TIME_FORMAT, \
+      GST_TIME_ARGS (timestamp), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)))
 
-  /* increase the refcount because we store it globally for others to use */
-  buf = gst_buffer_ref (buffer);
+#define check_buffer_duration(buffer,duration) \
+  fail_unless (GST_BUFFER_DURATION (buffer) == duration, \
+      "expected duration %" GST_TIME_FORMAT \
+      ", but got duration %" GST_TIME_FORMAT, \
+      GST_TIME_ARGS (duration), GST_TIME_ARGS (GST_BUFFER_DURATION (buffer)))
 
-  g_cond_signal (cond);
-
-  g_mutex_unlock (lock);
-
-  return TRUE;
-}
+static gboolean old_libtheora;
 
 static void
-start_pipeline (GstElement * bin, GstPad * pad)
+check_libtheora (void)
 {
-  GstStateChangeReturn ret;
-
-  id = gst_pad_add_buffer_probe (pad, G_CALLBACK (buffer_probe), NULL);
-
-  cond = g_cond_new ();
-  lock = g_mutex_new ();
-
-  ret = gst_element_set_state (bin, GST_STATE_PLAYING);
-  fail_if (ret == GST_STATE_CHANGE_FAILURE, "Could not start test pipeline");
-  if (ret == GST_STATE_CHANGE_ASYNC) {
-    ret = gst_element_get_state (bin, NULL, NULL, GST_CLOCK_TIME_NONE);
-    fail_if (ret != GST_STATE_CHANGE_SUCCESS, "Could not start test pipeline");
-  }
-}
-
-/* waits until the probe receives a buffer.  will catch every buffer */
-static GstBuffer *
-get_buffer (GstElement * bin, GstPad * pad)
-{
-  GstBuffer *ret;
-
-  g_mutex_lock (lock);
-
-  while (buf == NULL)
-    g_cond_wait (cond, lock);
-
-  ret = buf;
-  buf = NULL;
-
-  g_cond_signal (cond);
-
-  g_mutex_unlock (lock);
-
-  return ret;
-}
-
-static void
-stop_pipeline (GstElement * bin, GstPad * pad)
-{
-  GstStateChangeReturn ret;
-
-  g_mutex_lock (lock);
-  if (buf)
-    gst_buffer_unref (buf);
-  buf = NULL;
-  gst_pad_remove_buffer_probe (pad, (guint) id);
-  id = 0;
-  g_cond_signal (cond);
-  g_mutex_unlock (lock);
-
-  ret = gst_element_set_state (bin, GST_STATE_NULL);
-  fail_if (ret == GST_STATE_CHANGE_FAILURE, "Could not stop test pipeline");
-  if (ret == GST_STATE_CHANGE_ASYNC) {
-    ret = gst_element_get_state (bin, NULL, NULL, GST_CLOCK_TIME_NONE);
-    fail_if (ret != GST_STATE_CHANGE_SUCCESS, "Could not stop test pipeline");
-  }
-
-  g_mutex_lock (lock);
-  if (buf)
-    gst_buffer_unref (buf);
-  buf = NULL;
-  g_mutex_unlock (lock);
-
-  g_mutex_free (lock);
-  g_cond_free (cond);
-
-  lock = NULL;
-  cond = NULL;
-}
-
-static void
-check_buffer_is_header (GstBuffer * buffer, gboolean is_header)
-{
-  fail_unless (GST_BUFFER_FLAG_IS_SET (buffer,
-          GST_BUFFER_FLAG_IN_CAPS) == is_header,
-      "GST_BUFFER_IN_CAPS is set to %d but expected %d",
-      GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_IN_CAPS), is_header);
-}
-
-static void
-check_buffer_timestamp (GstBuffer * buffer, GstClockTime timestamp)
-{
-  fail_unless (GST_BUFFER_TIMESTAMP (buffer) == timestamp,
-      "expected timestamp %" GST_TIME_FORMAT
-      ", but got timestamp %" GST_TIME_FORMAT,
-      GST_TIME_ARGS (timestamp), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)));
-}
-
-static void
-check_buffer_duration (GstBuffer * buffer, GstClockTime duration)
-{
-  fail_unless (GST_BUFFER_DURATION (buffer) == duration,
-      "expected duration %" GST_TIME_FORMAT
-      ", but got duration %" GST_TIME_FORMAT,
-      GST_TIME_ARGS (duration), GST_TIME_ARGS (GST_BUFFER_DURATION (buffer)));
+  old_libtheora = (theora_version_number () <= 0x00030200);
 }
 
 static void
 check_buffer_granulepos (GstBuffer * buffer, gint64 granulepos)
 {
   GstClockTime clocktime;
+  int framecount;
+
+  /* With old versions of libtheora, the granulepos represented the
+   * start time, not end time. Adapt for that. */
+  if (old_libtheora) {
+    if (granulepos >> GRANULEPOS_SHIFT)
+      granulepos -= 1 << GRANULEPOS_SHIFT;
+    else if (granulepos)
+      granulepos -= 1;
+  }
 
   fail_unless (GST_BUFFER_OFFSET_END (buffer) == granulepos,
       "expected granulepos %" G_GUINT64_FORMAT
@@ -165,8 +82,10 @@ check_buffer_granulepos (GstBuffer * buffer, gint64 granulepos)
 
   /* contrary to what we record as TIMESTAMP, we can use OFFSET to check
    * the granulepos correctly here */
-  clocktime = gst_util_uint64_scale (GST_BUFFER_OFFSET_END (buffer), GST_SECOND,
-      FRAMERATE);
+  framecount = GST_BUFFER_OFFSET_END (buffer);
+  framecount = granulepos >> GRANULEPOS_SHIFT;
+  framecount += granulepos & ((1 << GRANULEPOS_SHIFT) - 1);
+  clocktime = gst_util_uint64_scale (framecount, GST_SECOND, FRAMERATE);
 
   fail_unless (clocktime == GST_BUFFER_OFFSET (buffer),
       "expected OFFSET set to clocktime %" GST_TIME_FORMAT
@@ -187,6 +106,15 @@ check_buffer_granulepos_from_starttime (GstBuffer * buffer,
   gint64 granulepos, expected, framecount;
 
   granulepos = GST_BUFFER_OFFSET_END (buffer);
+  /* Now convert to 'granulepos for start time', depending on libtheora 
+   * version */
+  if (!old_libtheora) {
+    if (granulepos & ((1 << GRANULEPOS_SHIFT) - 1))
+      granulepos -= 1;
+    else if (granulepos)
+      granulepos -= 1 << GRANULEPOS_SHIFT;
+  }
+
   framecount = granulepos >> GRANULEPOS_SHIFT;
   framecount += granulepos & ((1 << GRANULEPOS_SHIFT) - 1);
   expected = gst_util_uint64_scale (starttime, FRAMERATE, GST_SECOND);
@@ -195,7 +123,7 @@ check_buffer_granulepos_from_starttime (GstBuffer * buffer,
       "expected frame count %" G_GUINT64_FORMAT
       " or %" G_GUINT64_FORMAT
       ", but got frame count %" G_GUINT64_FORMAT,
-      expected, expected + 1, granulepos);
+      expected, expected + 1, framecount);
 }
 
 GST_START_TEST (test_granulepos_offset)
@@ -205,7 +133,6 @@ GST_START_TEST (test_granulepos_offset)
   gchar *pipe_str;
   GstBuffer *buffer;
   GError *error = NULL;
-  GstClockTime timestamp;
 
   pipe_str = g_strdup_printf ("videotestsrc timestamp-offset=%" G_GUINT64_FORMAT
       " ! video/x-raw-yuv,format=(fourcc)I420,framerate=10/1"
@@ -226,24 +153,24 @@ GST_START_TEST (test_granulepos_offset)
     gst_object_unref (sink);
   }
 
-  start_pipeline (bin, pad);
+  gst_buffer_straw_start_pipeline (bin, pad);
 
   /* header packets should have timestamp == NONE, granulepos 0, IN_CAPS */
-  buffer = get_buffer (bin, pad);
+  buffer = gst_buffer_straw_get_buffer (bin, pad);
   check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_granulepos (buffer, 0);
   check_buffer_is_header (buffer, TRUE);
   gst_buffer_unref (buffer);
 
-  buffer = get_buffer (bin, pad);
+  buffer = gst_buffer_straw_get_buffer (bin, pad);
   check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_granulepos (buffer, 0);
   check_buffer_is_header (buffer, TRUE);
   gst_buffer_unref (buffer);
 
-  buffer = get_buffer (bin, pad);
+  buffer = gst_buffer_straw_get_buffer (bin, pad);
   check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_granulepos (buffer, 0);
@@ -260,7 +187,7 @@ GST_START_TEST (test_granulepos_offset)
      * same value due to loss of precision with granulepos. theoraenc does
      * take care to timestamp correctly based on the offset of the input data
      * however, so it does do sub-granulepos timestamping. */
-    buffer = get_buffer (bin, pad);
+    buffer = gst_buffer_straw_get_buffer (bin, pad);
     last_granulepos = GST_BUFFER_OFFSET_END (buffer);
     check_buffer_timestamp (buffer, TIMESTAMP_OFFSET);
     /* don't really have a good way of checking duration... */
@@ -272,7 +199,7 @@ GST_START_TEST (test_granulepos_offset)
     gst_buffer_unref (buffer);
 
     /* check continuity with the next buffer */
-    buffer = get_buffer (bin, pad);
+    buffer = gst_buffer_straw_get_buffer (bin, pad);
     check_buffer_timestamp (buffer, next_timestamp);
     check_buffer_duration (buffer,
         gst_util_uint64_scale (GST_BUFFER_OFFSET_END (buffer), GST_SECOND,
@@ -284,7 +211,7 @@ GST_START_TEST (test_granulepos_offset)
     gst_buffer_unref (buffer);
   }
 
-  stop_pipeline (bin, pad);
+  gst_buffer_straw_stop_pipeline (bin, pad);
 
   gst_object_unref (pad);
   gst_object_unref (bin);
@@ -299,7 +226,6 @@ GST_START_TEST (test_continuity)
   gchar *pipe_str;
   GstBuffer *buffer;
   GError *error = NULL;
-  GstClockTime timestamp;
 
   pipe_str = g_strdup_printf ("videotestsrc"
       " ! video/x-raw-yuv,format=(fourcc)I420,framerate=10/1"
@@ -320,24 +246,24 @@ GST_START_TEST (test_continuity)
     gst_object_unref (sink);
   }
 
-  start_pipeline (bin, pad);
+  gst_buffer_straw_start_pipeline (bin, pad);
 
   /* header packets should have timestamp == NONE, granulepos 0 */
-  buffer = get_buffer (bin, pad);
+  buffer = gst_buffer_straw_get_buffer (bin, pad);
   check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_granulepos (buffer, 0);
   check_buffer_is_header (buffer, TRUE);
   gst_buffer_unref (buffer);
 
-  buffer = get_buffer (bin, pad);
+  buffer = gst_buffer_straw_get_buffer (bin, pad);
   check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_granulepos (buffer, 0);
   check_buffer_is_header (buffer, TRUE);
   gst_buffer_unref (buffer);
 
-  buffer = get_buffer (bin, pad);
+  buffer = gst_buffer_straw_get_buffer (bin, pad);
   check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
   check_buffer_granulepos (buffer, 0);
@@ -354,12 +280,12 @@ GST_START_TEST (test_continuity)
      * same value due to loss of precision with granulepos. theoraenc does
      * take care to timestamp correctly based on the offset of the input data
      * however, so it does do sub-granulepos timestamping. */
-    buffer = get_buffer (bin, pad);
+    buffer = gst_buffer_straw_get_buffer (bin, pad);
     last_granulepos = GST_BUFFER_OFFSET_END (buffer);
     check_buffer_timestamp (buffer, 0);
     /* plain division because I know the answer is exact */
     check_buffer_duration (buffer, GST_SECOND / 10);
-    check_buffer_granulepos (buffer, 0);
+    check_buffer_granulepos (buffer, 1 << GRANULEPOS_SHIFT);
     check_buffer_is_header (buffer, FALSE);
 
     next_timestamp = GST_BUFFER_DURATION (buffer);
@@ -367,16 +293,16 @@ GST_START_TEST (test_continuity)
     gst_buffer_unref (buffer);
 
     /* check continuity with the next buffer */
-    buffer = get_buffer (bin, pad);
+    buffer = gst_buffer_straw_get_buffer (bin, pad);
     check_buffer_timestamp (buffer, next_timestamp);
     check_buffer_duration (buffer, GST_SECOND / 10);
-    check_buffer_granulepos (buffer, 1);
+    check_buffer_granulepos (buffer, (1 << GRANULEPOS_SHIFT) | 1);
     check_buffer_is_header (buffer, FALSE);
 
     gst_buffer_unref (buffer);
   }
 
-  stop_pipeline (bin, pad);
+  gst_buffer_straw_stop_pipeline (bin, pad);
 
   gst_object_unref (pad);
   gst_object_unref (bin);
@@ -384,6 +310,122 @@ GST_START_TEST (test_continuity)
 
 GST_END_TEST;
 
+static gboolean
+drop_second_data_buffer (GstPad * droppad, GstBuffer * buffer, gpointer unused)
+{
+  return !(GST_BUFFER_OFFSET (buffer) == 1);
+}
+
+GST_START_TEST (test_discontinuity)
+{
+  GstElement *bin;
+  GstPad *pad, *droppad;
+  gchar *pipe_str;
+  GstBuffer *buffer;
+  GError *error = NULL;
+  guint drop_id;
+
+  pipe_str = g_strdup_printf ("videotestsrc"
+      " ! video/x-raw-yuv,format=(fourcc)I420,framerate=10/1"
+      " ! theoraenc ! fakesink name=fs0");
+
+  bin = gst_parse_launch (pipe_str, &error);
+  fail_unless (bin != NULL, "Error parsing pipeline: %s",
+      error ? error->message : "(invalid error)");
+  g_free (pipe_str);
+
+  /* the plan: same as test_continuity, but dropping a buffer and seeing if
+     theoraenc correctly notes the discontinuity */
+
+  /* get the pad to use to drop buffers */
+  {
+    GstElement *sink = gst_bin_get_by_name (GST_BIN (bin), "theoraenc0");
+
+    fail_unless (sink != NULL, "Could not get theoraenc out of bin");
+    droppad = gst_element_get_pad (sink, "sink");
+    fail_unless (droppad != NULL, "Could not get pad out of theoraenc");
+    gst_object_unref (sink);
+  }
+
+  /* get the pad */
+  {
+    GstElement *sink = gst_bin_get_by_name (GST_BIN (bin), "fs0");
+
+    fail_unless (sink != NULL, "Could not get fakesink out of bin");
+    pad = gst_element_get_pad (sink, "sink");
+    fail_unless (pad != NULL, "Could not get pad out of fakesink");
+    gst_object_unref (sink);
+  }
+
+  drop_id = gst_pad_add_buffer_probe (droppad,
+      G_CALLBACK (drop_second_data_buffer), NULL);
+  gst_buffer_straw_start_pipeline (bin, pad);
+
+  /* header packets should have timestamp == NONE, granulepos 0 */
+  buffer = gst_buffer_straw_get_buffer (bin, pad);
+  check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
+  check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
+  check_buffer_granulepos (buffer, 0);
+  check_buffer_is_header (buffer, TRUE);
+  gst_buffer_unref (buffer);
+
+  buffer = gst_buffer_straw_get_buffer (bin, pad);
+  check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
+  check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
+  check_buffer_granulepos (buffer, 0);
+  check_buffer_is_header (buffer, TRUE);
+  gst_buffer_unref (buffer);
+
+  buffer = gst_buffer_straw_get_buffer (bin, pad);
+  check_buffer_timestamp (buffer, GST_CLOCK_TIME_NONE);
+  check_buffer_duration (buffer, GST_CLOCK_TIME_NONE);
+  check_buffer_granulepos (buffer, 0);
+  check_buffer_is_header (buffer, TRUE);
+  gst_buffer_unref (buffer);
+
+  {
+    buffer = gst_buffer_straw_get_buffer (bin, pad);
+    check_buffer_timestamp (buffer, 0);
+    /* plain division because I know the answer is exact */
+    check_buffer_duration (buffer, GST_SECOND / 10);
+    check_buffer_granulepos (buffer, 1 << GRANULEPOS_SHIFT);
+    check_buffer_is_header (buffer, FALSE);
+    fail_if (GST_BUFFER_IS_DISCONT (buffer), "expected continuous buffer yo");
+    gst_buffer_unref (buffer);
+
+    /* check discontinuity with the next buffer */
+    buffer = gst_buffer_straw_get_buffer (bin, pad);
+    check_buffer_duration (buffer, GST_SECOND / 10);
+    /* After a discont, we'll always get a keyframe, so this one should be 
+     * 3<<GRANULEPOS_SHIFT */
+    check_buffer_granulepos (buffer, 3 << GRANULEPOS_SHIFT);
+    check_buffer_is_header (buffer, FALSE);
+    fail_unless (GST_BUFFER_IS_DISCONT (buffer),
+        "expected discontinuous buffer yo");
+    gst_buffer_unref (buffer);
+
+    /* Then the buffer after that should be continuous */
+    buffer = gst_buffer_straw_get_buffer (bin, pad);
+    fail_if (GST_BUFFER_IS_DISCONT (buffer), "expected continuous buffer yo");
+    /* plain division because I know the answer is exact */
+    check_buffer_duration (buffer, GST_SECOND / 10);
+    check_buffer_granulepos (buffer, (3 << GRANULEPOS_SHIFT) | 1);
+    check_buffer_is_header (buffer, FALSE);
+    gst_buffer_unref (buffer);
+  }
+
+  gst_buffer_straw_stop_pipeline (bin, pad);
+  gst_pad_remove_buffer_probe (droppad, drop_id);
+
+  gst_object_unref (droppad);
+  gst_object_unref (pad);
+  gst_object_unref (bin);
+}
+
+GST_END_TEST;
+
+#endif /* #ifndef GST_DISABLE_PARSE */
+
 Suite *
 theoraenc_suite (void)
 {
@@ -391,8 +433,14 @@ theoraenc_suite (void)
   TCase *tc_chain = tcase_create ("general");
 
   suite_add_tcase (s, tc_chain);
+
+  check_libtheora ();
+
+#ifndef GST_DISABLE_PARSE
   tcase_add_test (tc_chain, test_granulepos_offset);
   tcase_add_test (tc_chain, test_continuity);
+  tcase_add_test (tc_chain, test_discontinuity);
+#endif
 
   return s;
 }
