@@ -848,6 +848,24 @@ make_playerbin_pipeline (const gchar * location)
   return player;
 }
 
+static GstElement *
+make_playerbin2_pipeline (const gchar * location)
+{
+  GstElement *player;
+
+  player = gst_element_factory_make ("playbin2", "player");
+  g_assert (player);
+
+  g_object_set (G_OBJECT (player), "uri", location, NULL);
+
+  seekable_elements = g_list_prepend (seekable_elements, player);
+
+  /* force element seeking on this pipeline */
+  elem_seek = TRUE;
+
+  return player;
+}
+
 #ifndef GST_DISABLE_PARSE
 static GstElement *
 make_parselaunch_pipeline (const gchar * description)
@@ -891,6 +909,7 @@ static Pipeline pipelines[] = {
 #ifndef GST_DISABLE_PARSE
   {"parse-launch", make_parselaunch_pipeline},
 #endif
+  {"playerbin2", make_playerbin2_pipeline},
   {NULL, NULL},
 };
 
@@ -1333,10 +1352,11 @@ stop_cb (GtkButton * button, gpointer data)
     if (ret == GST_STATE_CHANGE_FAILURE)
       goto failed;
 
-    set_scale (0.0);
-
     state = GST_STATE_READY;
 
+    set_update_scale (FALSE);
+    set_scale (0.0);
+
     /* if one uses parse_launch, play, stop and play again it fails as all the
      * pads after the demuxer can't be reconnected
      */
@@ -1454,23 +1474,6 @@ message_received (GstBus * bus, GstMessage * message, GstPipeline * pipeline)
   }
 }
 
-static void
-msg_eos (GstBus * bus, GstMessage * message, GstPipeline * pipeline)
-{
-  GstStateChangeReturn ret;
-
-  GST_DEBUG ("position is %" GST_TIME_FORMAT, GST_TIME_ARGS (position));
-
-  g_print ("READY pipeline\n");
-  ret = gst_element_set_state (GST_ELEMENT (pipeline), GST_STATE_READY);
-  if (ret == GST_STATE_CHANGE_FAILURE)
-    g_print ("READY failed\n");
-
-  set_scale (0.0);
-
-  state = GST_STATE_READY;
-}
-
 static void
 msg_state_changed (GstBus * bus, GstMessage * message, GstPipeline * pipeline)
 {
@@ -1502,15 +1505,14 @@ msg_segment_done (GstBus * bus, GstMessage * message, GstPipeline * pipeline)
   GstFormat format;
 
   GST_DEBUG ("position is %" GST_TIME_FORMAT, GST_TIME_ARGS (position));
-  format = GST_FORMAT_TIME;
   gst_message_parse_segment_done (message, &format, &position);
   GST_DEBUG ("end of segment at %" GST_TIME_FORMAT, GST_TIME_ARGS (position));
 
-  set_update_scale (FALSE);
-
-  flags = GST_SEEK_FLAG_SEGMENT;
-  if (flush_seek)
-    flags |= GST_SEEK_FLAG_FLUSH;
+  flags = 0;
+  /* in the segment-done callback we never flush as this would not make sense
+   * for seamless playback. */
+  if (loop_seek)
+    flags = GST_SEEK_FLAG_SEGMENT;
 
   s_event = gst_event_new_seek (rate,
       GST_FORMAT_TIME, flags, GST_SEEK_TYPE_SET, G_GINT64_CONSTANT (0),
@@ -1520,16 +1522,8 @@ msg_segment_done (GstBus * bus, GstMessage * message, GstPipeline * pipeline)
       rate, GST_TIME_ARGS (duration));
 
   res = send_event (s_event);
-  if (res) {
-    if (flush_seek) {
-      gst_pipeline_set_new_stream_time (GST_PIPELINE (pipeline), 0);
-      gst_element_get_state (GST_ELEMENT (pipeline), NULL, NULL, SEEK_TIMEOUT);
-    }
-  } else
+  if (!res)
     g_print ("segment seek failed\n");
-
-  position = 0;
-  set_update_scale (TRUE);
 }
 
 static void
@@ -1619,7 +1613,7 @@ main (int argc, char **argv)
   flush_checkbox = gtk_check_button_new_with_label ("Flush");
   scrub_checkbox = gtk_check_button_new_with_label ("Scrub");
   play_scrub_checkbox = gtk_check_button_new_with_label ("Play Scrub");
-  rate_spinbutton = gtk_spin_button_new_with_range (-10, 10, 0.1);
+  rate_spinbutton = gtk_spin_button_new_with_range (-100, 100, 0.1);
   rate_label = gtk_label_new ("Rate");
 
   gtk_tooltips_set_tip (tips, accurate_checkbox,
@@ -1648,6 +1642,10 @@ main (int argc, char **argv)
       GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.00, 100.0, 0.1, 1.0, 1.0));
   hscale = gtk_hscale_new (adjustment);
   gtk_scale_set_digits (GTK_SCALE (hscale), 2);
+#if GTK_CHECK_VERSION(2,12,0)
+  gtk_range_set_show_fill_level (GTK_RANGE (hscale), TRUE);
+  gtk_range_set_fill_level (GTK_RANGE (hscale), 100.0);
+#endif
   gtk_range_set_update_policy (GTK_RANGE (hscale), GTK_UPDATE_CONTINUOUS);
 
   gtk_signal_connect (GTK_OBJECT (hscale),
@@ -1715,7 +1713,6 @@ main (int argc, char **argv)
     bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
     gst_bus_add_signal_watch_full (bus, G_PRIORITY_HIGH);
 
-    g_signal_connect (bus, "message::eos", (GCallback) msg_eos, pipeline);
     g_signal_connect (bus, "message::state-changed",
         (GCallback) msg_state_changed, pipeline);
     g_signal_connect (bus, "message::segment-done",
@@ -1735,10 +1732,6 @@ main (int argc, char **argv)
         pipeline);
     g_signal_connect (bus, "message::segment-done",
         (GCallback) message_received, pipeline);
-    /*g_signal_connect (bus, "message::state-changed",
-       (GCallback) message_received, pipeline);
-     */
-
   }
   gtk_main ();
 
